@@ -18,7 +18,6 @@ import uuid
 
 from oslo_log import log
 from oslo_utils import strutils
-from oslo_utils import timeutils
 import pecan
 from pecan import rest
 import six
@@ -46,8 +45,10 @@ def arg_to_list(value):
     return []
 
 
-def abort(status_code=None, detail='', headers=None, comment=None, **kw):
+def abort(status_code, detail='', headers=None, comment=None, **kw):
     """Like pecan.abort, but make sure detail is a string."""
+    if status_code == 404 and not detail:
+        raise RuntimeError("http code 404 must have 'detail' set")
     return pecan.abort(status_code, six.text_type(detail),
                        headers, comment, **kw)
 
@@ -112,6 +113,7 @@ def enforce(rule, target):
 
 
 def set_resp_location_hdr(location):
+    location = '%s%s' % (pecan.request.script_name, location)
     # NOTE(sileht): according the pep-3333 the headers must be
     # str in py2 and py3 even this is not the same thing in both
     # version
@@ -122,18 +124,23 @@ def set_resp_location_hdr(location):
     pecan.response.headers['Location'] = location
 
 
-def deserialize(schema, required=True):
+def deserialize():
     mime_type, options = werkzeug.http.parse_options_header(
         pecan.request.headers.get('Content-Type'))
     if mime_type != "application/json":
         abort(415)
     try:
-        params = json.loads(pecan.request.body.decode(
-            options.get('charset', 'ascii')))
+        params = json.load(pecan.request.body_file_raw,
+                           encoding=options.get('charset', 'ascii'))
     except Exception as e:
         abort(400, "Unable to decode body: " + six.text_type(e))
+    return params
+
+
+def deserialize_and_validate(schema, required=True):
     try:
-        return voluptuous.Schema(schema, required=required)(params)
+        return voluptuous.Schema(schema, required=required)(
+            deserialize())
     except voluptuous.Error as e:
         abort(400, "Invalid input: %s" % e)
 
@@ -144,20 +151,6 @@ def Timestamp(v):
     return utils.to_timestamp(v)
 
 
-def convert_metric_list(metrics, created_by_user_id, created_by_project_id):
-    # Replace an archive policy as value for an metric by a brand
-    # a new metric
-    new_metrics = {}
-    for k, v in six.iteritems(metrics):
-        if isinstance(v, uuid.UUID):
-            new_metrics[k] = v
-        else:
-            new_metrics[k] = str(MetricsController.create_metric(
-                created_by_user_id, created_by_project_id,
-                v['archive_policy_name'])['id'])
-    return new_metrics
-
-
 def PositiveOrNullInt(value):
     value = int(value)
     if value < 0:
@@ -199,6 +192,32 @@ def get_details(params):
     return get_header_option('details', params)
 
 
+RESOURCE_DEFAULT_PAGINATION = ['revision_start:asc',
+                               'started_at:asc']
+
+
+def get_pagination_options(params, default):
+    max_limit = pecan.request.conf.api.max_limit
+    limit = params.get('limit', max_limit)
+    marker = params.get('marker')
+    sorts = params.get('sort', default)
+    if not isinstance(sorts, list):
+        sorts = [sorts]
+
+    try:
+        limit = int(limit)
+        if limit <= 0:
+            raise ValueError
+    except ValueError:
+        abort(400, "Invalid 'limit' value: %s" % params.get('limit'))
+
+    limit = min(limit, max_limit)
+
+    return {'limit': limit,
+            'marker': marker,
+            'sorts': sorts}
+
+
 def ValidAggMethod(value):
     value = six.text_type(value)
     if value in archive_policy.ArchivePolicy.VALID_AGGREGATION_METHODS_VALUES:
@@ -206,7 +225,37 @@ def ValidAggMethod(value):
     raise ValueError("Invalid aggregation method")
 
 
+class ArchivePolicyController(rest.RestController):
+    def __init__(self, archive_policy):
+        self.archive_policy = archive_policy
+
+    @pecan.expose('json')
+    def get(self):
+        ap = pecan.request.indexer.get_archive_policy(self.archive_policy)
+        if ap:
+            enforce("get archive policy", ap)
+            return ap
+        abort(404, indexer.NoSuchArchivePolicy(self.archive_policy))
+
+    @pecan.expose()
+    def delete(self):
+        # NOTE(jd) I don't think there's any point in fetching and passing the
+        # archive policy here, as the rule is probably checking the actual role
+        # of the user, not the content of the AP.
+        enforce("delete archive policy", {})
+        try:
+            pecan.request.indexer.delete_archive_policy(self.archive_policy)
+        except indexer.NoSuchArchivePolicy as e:
+            abort(404, e)
+        except indexer.ArchivePolicyInUse as e:
+            abort(400, e)
+
+
 class ArchivePoliciesController(rest.RestController):
+    @pecan.expose()
+    def _lookup(self, archive_policy, *remainder):
+        return ArchivePolicyController(archive_policy), remainder
+
     @pecan.expose('json')
     def post(self):
         # NOTE(jd): Initialize this one at run-time because we rely on conf
@@ -227,7 +276,7 @@ class ArchivePoliciesController(rest.RestController):
                 }], voluptuous.Length(min=1)),
             })
 
-        body = deserialize(ArchivePolicySchema)
+        body = deserialize_and_validate(ArchivePolicySchema)
         # Validate the data
         try:
             ap = archive_policy.ArchivePolicy.from_dict(body)
@@ -239,39 +288,18 @@ class ArchivePoliciesController(rest.RestController):
         except indexer.ArchivePolicyAlreadyExists as e:
             abort(409, e)
 
-        location = "/v1/archive_policy/" + ap['name']
+        location = "/v1/archive_policy/" + ap.name
         set_resp_location_hdr(location)
         pecan.response.status = 201
         return ap
 
-    @pecan.expose('json')
-    def get_one(self, id):
-        ap = pecan.request.indexer.get_archive_policy(id)
-        if ap:
-            enforce("get archive policy", ap)
-            return ap
-        abort(404)
-
     @pecan.expose('json')
     def get_all(self):
         enforce("list archive policy", {})
         return pecan.request.indexer.list_archive_policies()
 
-    @pecan.expose()
-    def delete(self, name):
-        try:
-            pecan.request.indexer.delete_archive_policy(name)
-        except indexer.NoSuchArchivePolicy as e:
-            abort(404, e)
-        except indexer.ArchivePolicyInUse as e:
-            abort(400, e)
-
 
 class ArchivePolicyRulesController(rest.RestController):
-    _custom_actions = {
-        'measures': ['GET', 'POST']
-    }
-
     @pecan.expose('json')
     def post(self):
         enforce("create archive policy rule", {})
@@ -281,7 +309,7 @@ class ArchivePolicyRulesController(rest.RestController):
             voluptuous.Required("archive_policy_name"): six.text_type,
             })
 
-        body = deserialize(ArchivePolicyRuleSchema)
+        body = deserialize_and_validate(ArchivePolicyRuleSchema)
         enforce("create archive policy rule", body)
         try:
             ap = pecan.request.indexer.create_archive_policy_rule(
@@ -291,7 +319,7 @@ class ArchivePolicyRulesController(rest.RestController):
         except indexer.ArchivePolicyRuleAlreadyExists as e:
             abort(409, e)
 
-        location = "/v1/archive_policy_rule/" + ap['name']
+        location = "/v1/archive_policy_rule/" + ap.name
         set_resp_location_hdr(location)
         pecan.response.status = 201
         return ap
@@ -302,7 +330,7 @@ class ArchivePolicyRulesController(rest.RestController):
         if ap:
             enforce("get archive policy rule", ap)
             return ap
-        abort(404)
+        abort(404, indexer.NoSuchArchivePolicyRule(name))
 
     @pecan.expose('json')
     def get_all(self):
@@ -311,6 +339,10 @@ class ArchivePolicyRulesController(rest.RestController):
 
     @pecan.expose()
     def delete(self, name):
+        # NOTE(jd) I don't think there's any point in fetching and passing the
+        # archive policy rule here, as the rule is probably checking the actual
+        # role of the user, not the content of the AP rule.
+        enforce("delete archive policy rule", {})
         try:
             pecan.request.indexer.delete_archive_policy_rule(name)
         except indexer.NoSuchArchivePolicyRule as e:
@@ -352,6 +384,10 @@ class AggregatedMetricController(rest.RestController):
     def get_cross_metric_measures_from_objs(metrics, start=None, stop=None,
                                             aggregation='mean',
                                             needed_overlap=100.0):
+        try:
+            needed_overlap = float(needed_overlap)
+        except ValueError:
+            abort(400, 'needed_overlap must be a number')
 
         if (aggregation
            not in archive_policy.ArchivePolicy.VALID_AGGREGATION_METHODS):
@@ -368,21 +404,17 @@ class AggregatedMetricController(rest.RestController):
             if len(metrics) == 1:
                 # NOTE(sileht): don't do the aggregation if we only have one
                 # metric
-                # NOTE(jd): the archive policy is None as it's not really used
-                # and it has a cost to request it from the indexer
                 measures = pecan.request.storage.get_measures(
                     metrics[0], start, stop, aggregation)
             else:
-                # NOTE(jd): the archive policy is None as it's not really used
-                # and it has a cost to request it from the indexer
                 measures = pecan.request.storage.get_cross_metric_measures(
                     metrics, start, stop, aggregation, needed_overlap)
             # Replace timestamp keys by their string versions
-            return [(timeutils.isotime(timestamp, subsecond=True), offset, v)
+            return [(timestamp.isoformat(), offset, v)
                     for timestamp, offset, v in measures]
-        except storage.MetricUnaggregatable:
-            abort(400, "One of the metric to aggregated doesn't have "
-                  "matching granularity")
+        except storage.MetricUnaggregatable as e:
+            abort(400, ("One of the metrics being aggregated doesn't have "
+                        "matching granularity: %s") % str(e))
         except storage.MetricDoesNotExist as e:
             abort(404, e)
         except storage.AggregationDoesNotExist as e:
@@ -400,11 +432,22 @@ class MetricController(rest.RestController):
                                          invoke_on_load=True)
         self.custom_agg = dict((x.name, x.obj) for x in mgr)
 
-    Measures = voluptuous.Schema([{
-        voluptuous.Required("timestamp"):
-        Timestamp,
-        voluptuous.Required("value"): voluptuous.Any(float, int),
-    }])
+    @staticmethod
+    def to_measure(m):
+        # NOTE(sileht): we do the input validation
+        # during the iteration for not loop just for this
+        # and don't use voluptuous for performance reason
+        try:
+            value = float(m['value'])
+        except Exception:
+            abort(400, "Invalid input for a value")
+
+        try:
+            timestamp = utils.to_timestamp(m['timestamp'])
+        except Exception:
+            abort(400, "Invalid input for a timestamp")
+
+        return storage.Measure(timestamp, value)
 
     def enforce_metric(self, rule):
         enforce(rule, json.to_primitive(self.metric))
@@ -417,24 +460,17 @@ class MetricController(rest.RestController):
     @pecan.expose()
     def post_measures(self):
         self.enforce_metric("post measures")
-        try:
+        params = deserialize()
+        if not isinstance(params, list):
+            abort(400, "Invalid input for measures")
+        if params:
             pecan.request.storage.add_measures(
-                self.metric,
-                (storage.Measure(
-                    m['timestamp'],
-                    m['value']) for m in deserialize(self.Measures)))
-        except storage.MetricDoesNotExist as e:
-            abort(404, e)
-        except storage.NoDeloreanAvailable as e:
-            abort(400,
-                  "The measure for %s is too old considering the "
-                  "archive policy used by this metric. "
-                  "It can only go back to %s."
-                  % (e.bad_timestamp, e.first_timestamp))
+                self.metric, six.moves.map(self.to_measure, params))
+        pecan.response.status = 202
 
     @pecan.expose('json')
-    @pecan.expose('measures.j2')
-    def get_measures(self, start=None, stop=None, aggregation='mean', **param):
+    def get_measures(self, start=None, stop=None, aggregation='mean',
+                     granularity=None, **param):
         self.enforce_metric("get measures")
         if not (aggregation
                 in archive_policy.ArchivePolicy.VALID_AGGREGATION_METHODS
@@ -465,12 +501,10 @@ class MetricController(rest.RestController):
                     start, stop, **param)
             else:
                 measures = pecan.request.storage.get_measures(
-                    # NOTE(jd) We don't set the archive policy in the object
-                    # here because it's not used; but we could do it if needed
-                    # by requesting the metric details from the indexer
-                    self.metric, start, stop, aggregation)
+                    self.metric, start, stop, aggregation,
+                    int(granularity) if granularity is not None else None)
             # Replace timestamp keys by their string versions
-            return [(timeutils.isotime(timestamp, subsecond=True), offset, v)
+            return [(timestamp.isoformat(), offset, v)
                     for timestamp, offset, v in measures]
         except storage.MetricDoesNotExist as e:
             abort(404, e)
@@ -483,10 +517,9 @@ class MetricController(rest.RestController):
     def delete(self):
         self.enforce_metric("delete metric")
         try:
-            pecan.request.storage.delete_metric(self.metric)
-        except storage.MetricDoesNotExist as e:
+            pecan.request.indexer.delete_metric(self.metric.id)
+        except indexer.NoSuchMetric as e:
             abort(404, e)
-        pecan.request.indexer.delete_metric(self.metric.id)
 
 
 def UUID(value):
@@ -496,14 +529,6 @@ def UUID(value):
         raise ValueError(e)
 
 
-MetricSchemaDefinition = {
-    "user_id": UUID,
-    "project_id": UUID,
-    "archive_policy_name": six.text_type,
-    "name": six.text_type,
-}
-
-
 class MetricsController(rest.RestController):
     @staticmethod
     @pecan.expose()
@@ -511,59 +536,74 @@ class MetricsController(rest.RestController):
         try:
             metric_id = uuid.UUID(id)
         except ValueError:
-            abort(404)
+            abort(404, indexer.NoSuchMetric(id))
         metrics = pecan.request.indexer.get_metrics([metric_id])
         if not metrics:
-            abort(404)
+            abort(404, indexer.NoSuchMetric(id))
         return MetricController(metrics[0]), remainder
 
-    Metric = voluptuous.Schema(MetricSchemaDefinition)
+    _MetricSchema = voluptuous.Schema({
+        "user_id": UUID,
+        "project_id": UUID,
+        "archive_policy_name": six.text_type,
+        "name": six.text_type,
+    })
 
-    @staticmethod
-    def create_metric(created_by_user_id, created_by_project_id,
-                      archive_policy_name=None, name=None,
-                      user_id=None, project_id=None):
-        enforce("create metric", {
-            "created_by_user_id": created_by_user_id,
-            "created_by_project_id": created_by_project_id,
-            "user_id": user_id,
-            "project_id": project_id,
-            "archive_policy_name": archive_policy_name,
-            "name": name
-        })
-        id = uuid.uuid4()
-        policy = None
-        if name is not None and archive_policy_name is None:
+    # NOTE(jd) Define this method as it was a voluptuous schema – it's just a
+    # smarter version of a voluptuous schema, no?
+    @classmethod
+    def MetricSchema(cls, definition):
+        # First basic validation
+        definition = cls._MetricSchema(definition)
+        archive_policy_name = definition.get('archive_policy_name')
+
+        name = definition.get('name')
+        if archive_policy_name is None:
             rules = pecan.request.indexer.list_archive_policy_rules()
             for rule in rules:
-                if fnmatch.fnmatch(name, rule['metric_pattern']):
-                    policy = pecan.request.indexer.get_archive_policy(
-                        rule['archive_policy_name'])
+                if fnmatch.fnmatch(name or "", rule.metric_pattern):
+                    ap = pecan.request.indexer.get_archive_policy(
+                        rule.archive_policy_name)
+                    definition['archive_policy_name'] = ap.name
                     break
             else:
-                abort(400, "No archive policy name specified and no archive"
-                           " policy rule found matching the metric name %s"
-                           % name)
-        else:
-            policy = pecan.request.indexer.get_archive_policy(
-                archive_policy_name)
-            if policy is None:
-                abort(400, "Unknown archive policy %s" % archive_policy_name)
-        m = pecan.request.indexer.create_metric(
-            id,
-            created_by_user_id, created_by_project_id,
-            archive_policy_name=policy.name, name=name, details=True)
-        pecan.request.storage.create_metric(storage.Metric(id, policy))
-        return m
+                # NOTE(jd) Since this is a schema-like function, we
+                # should/could raise ValueError, but if we do so, voluptuous
+                # just returns a "invalid value" with no useful message – so we
+                # prefer to use abort() to make sure the user has the right
+                # error message
+                abort(400, "No archive policy name specified "
+                      "and no archive policy rule found matching "
+                      "the metric name %s" % name)
+
+        user_id, project_id = get_user_and_project()
+
+        enforce("create metric", {
+            "created_by_user_id": user_id,
+            "created_by_project_id": project_id,
+            "user_id": definition.get('user_id'),
+            "project_id": definition.get('project_id'),
+            "archive_policy_name": archive_policy_name,
+            "name": name,
+        })
+
+        return definition
 
     @pecan.expose('json')
     def post(self):
         user, project = get_user_and_project()
-        body = deserialize(self.Metric)
-        metric_info = self.create_metric(user, project, **body)
-        set_resp_location_hdr("/v1/metric/" + str(metric_info['id']))
+        body = deserialize_and_validate(self.MetricSchema)
+        try:
+            m = pecan.request.indexer.create_metric(
+                uuid.uuid4(),
+                user, project,
+                name=body.get('name'),
+                archive_policy_name=body['archive_policy_name'])
+        except indexer.NoSuchArchivePolicy as e:
+            abort(400, e)
+        set_resp_location_hdr("/v1/metric/" + str(m.id))
         pecan.response.status = 201
-        return metric_info
+        return m
 
     @staticmethod
     @pecan.expose('json')
@@ -586,12 +626,24 @@ class MetricsController(rest.RestController):
             user_id, project_id)
 
 
-Metrics = voluptuous.Schema({
+_MetricsSchema = voluptuous.Schema({
     six.text_type: voluptuous.Any(UUID,
-                                  MetricsController.Metric),
+                                  MetricsController.MetricSchema),
 })
 
 
+def MetricsSchema(data):
+    # NOTE(jd) Before doing any kind of validation, copy the metric name
+    # into the metric definition. This is required so we have the name
+    # available when doing the metric validation with its own MetricSchema,
+    # and so we can do things such as applying archive policy rules.
+    if isinstance(data, dict):
+        for metric_name, metric_def in six.iteritems(data):
+            if isinstance(metric_def, dict):
+                metric_def['name'] = metric_name
+    return _MetricsSchema(data)
+
+
 class NamedMetricController(rest.RestController):
     def __init__(self, resource_id, resource_type):
         self.resource_id = resource_id
@@ -599,35 +651,81 @@ class NamedMetricController(rest.RestController):
 
     @pecan.expose()
     def _lookup(self, name, *remainder):
-        # TODO(jd) There might be an slight optimization to do by using a
-        # dedicated driver method rather than get_resource, which might be
-        # heavier.
-        resource = pecan.request.indexer.get_resource(
-            'generic', self.resource_id, with_metrics=True)
+        m = pecan.request.indexer.list_metrics(details=True,
+                                               name=name,
+                                               resource_id=self.resource_id)
+        if m:
+            return MetricController(m[0]), remainder
+
+        resource = pecan.request.indexer.get_resource(self.resource_type,
+                                                      self.resource_id)
         if resource:
-            m = resource.get_metric(name)
-            if m:
-                return MetricController(m), remainder
-        abort(404)
+            abort(404, indexer.NoSuchMetric(name))
+        else:
+            abort(404, indexer.NoSuchResource(self.resource_id))
 
     @pecan.expose()
     def post(self):
         resource = pecan.request.indexer.get_resource(
             self.resource_type, self.resource_id)
+        if not resource:
+            abort(404, indexer.NoSuchResource(self.resource_id))
         enforce("update resource", resource)
-        user, project = get_user_and_project()
-        metrics = convert_metric_list(deserialize(Metrics), user, project)
+        metrics = deserialize_and_validate(MetricsSchema)
         try:
             pecan.request.indexer.update_resource(
                 self.resource_type, self.resource_id, metrics=metrics,
                 append_metrics=True)
-        except (indexer.NoSuchMetric, ValueError) as e:
+        except (indexer.NoSuchMetric,
+                indexer.NoSuchArchivePolicy,
+                ValueError) as e:
             abort(400, e)
         except indexer.NamedMetricAlreadyExists as e:
             abort(409, e)
         except indexer.NoSuchResource as e:
             abort(404, e)
 
+    @pecan.expose('json')
+    def get_all(self):
+        resource = pecan.request.indexer.get_resource(
+            self.resource_type, self.resource_id)
+        if not resource:
+            abort(404, indexer.NoSuchResource(self.resource_id))
+        enforce("get resource", resource)
+        return pecan.request.indexer.list_metrics(resource_id=self.resource_id)
+
+
+class ResourceHistoryController(rest.RestController):
+    def __init__(self, resource_id, resource_type):
+        self.resource_id = resource_id
+        self.resource_type = resource_type
+
+    @pecan.expose('json')
+    def get(self, **kwargs):
+        details = get_details(kwargs)
+        pagination_opts = get_pagination_options(
+            kwargs, RESOURCE_DEFAULT_PAGINATION)
+
+        resource = pecan.request.indexer.get_resource(
+            self.resource_type, self.resource_id)
+        if not resource:
+            abort(404, "foo")
+
+        enforce("get resource", resource)
+
+        try:
+            # FIXME(sileht): next API version should returns
+            # {'resources': [...], 'links': [ ... pagination rel ...]}
+            return pecan.request.indexer.list_resources(
+                self.resource_type,
+                attribute_filter={"=": {"id": self.resource_id}},
+                details=details,
+                history=True,
+                **pagination_opts
+            )
+        except indexer.IndexerException as e:
+            abort(400, e)
+
 
 def etag_precondition_check(obj):
     etag, lastmodified = obj.etag, obj.lastmodified
@@ -662,12 +760,12 @@ def etag_set_headers(obj):
 
 def ResourceSchema(schema):
     base_schema = {
-        "id": UUID,
+        "id": utils.ResourceUUID,
         voluptuous.Optional('started_at'): Timestamp,
         voluptuous.Optional('ended_at'): Timestamp,
         voluptuous.Optional('user_id'): voluptuous.Any(None, UUID),
         voluptuous.Optional('project_id'): voluptuous.Any(None, UUID),
-        voluptuous.Optional('metrics'): Metrics,
+        voluptuous.Optional('metrics'): MetricsSchema,
     }
     base_schema.update(schema)
     return base_schema
@@ -680,13 +778,14 @@ class GenericResourceController(rest.RestController):
 
     def __init__(self, id):
         try:
-            self.id = uuid.UUID(id)
+            self.id = utils.ResourceUUID(id)
         except ValueError:
-            abort(404)
-        self.metric = NamedMetricController(id, self._resource_type)
+            abort(404, indexer.NoSuchResource(id))
+        self.metric = NamedMetricController(str(self.id), self._resource_type)
+        self.history = ResourceHistoryController(str(self.id),
+                                                 self._resource_type)
 
     @pecan.expose('json')
-    @pecan.expose('resources.j2')
     def get(self):
         resource = pecan.request.indexer.get_resource(
             self._resource_type, self.id, with_metrics=True)
@@ -695,19 +794,23 @@ class GenericResourceController(rest.RestController):
             etag_precondition_check(resource)
             etag_set_headers(resource)
             return resource
-        abort(404)
+        abort(404, indexer.NoSuchResource(self.id))
 
     @pecan.expose('json')
-    @pecan.expose('resources.j2')
     def patch(self):
         resource = pecan.request.indexer.get_resource(
             self._resource_type, self.id)
         if not resource:
-            abort(404)
+            abort(404, indexer.NoSuchResource(self.id))
         enforce("update resource", resource)
         etag_precondition_check(resource)
 
-        body = deserialize(self.Resource, required=False)
+        body = deserialize_and_validate(self.Resource, required=False)
+
+        if not self._resource_need_update(resource, body):
+            # No need to go further, we assume the db resource
+            # doesn't change between the get and update
+            return resource
         if len(body) == 0:
             etag_set_headers(resource)
             return resource
@@ -715,12 +818,12 @@ class GenericResourceController(rest.RestController):
         try:
             if 'metrics' in body:
                 user, project = get_user_and_project()
-                body['metrics'] = convert_metric_list(
-                    body['metrics'], user, project)
             resource = pecan.request.indexer.update_resource(
                 self._resource_type,
                 self.id, **body)
-        except (indexer.NoSuchMetric, ValueError) as e:
+        except (indexer.NoSuchMetric,
+                indexer.NoSuchArchivePolicy,
+                ValueError) as e:
             abort(400, e)
         except indexer.NoSuchResource as e:
             abort(404, e)
@@ -728,17 +831,13 @@ class GenericResourceController(rest.RestController):
         return resource
 
     @staticmethod
-    def _delete_metrics(metrics):
-        for metric in metrics:
-            enforce("delete metric", metric)
-        for metric in metrics:
-            try:
-                pecan.request.storage.delete_metric(metric)
-            except Exception:
-                LOG.error(
-                    "Unable to delete metric `%s' from storage, "
-                    "you will need to delete it manually" % metric.id,
-                    exc_info=True)
+    def _resource_need_update(resource, new_attributes):
+        if 'metrics' in new_attributes:
+            return True
+        for k, v in new_attributes.items():
+            if getattr(resource, k) != v:
+                return True
+        return False
 
     @pecan.expose()
     def delete(self):
@@ -749,9 +848,7 @@ class GenericResourceController(rest.RestController):
         enforce("delete resource", resource)
         etag_precondition_check(resource)
         try:
-            pecan.request.indexer.delete_resource(
-                self.id,
-                delete_metrics=self._delete_metrics)
+            pecan.request.indexer.delete_resource(self.id)
         except indexer.NoSuchResource as e:
             abort(404, e)
 
@@ -760,11 +857,27 @@ class SwiftAccountResourceController(GenericResourceController):
     _resource_type = 'swift_account'
 
 
+class InstanceDiskResourceController(GenericResourceController):
+    _resource_type = 'instance_disk'
+    Resource = ResourceSchema({
+        "name": six.text_type,
+        "instance_id": UUID,
+    })
+
+
+class InstanceNetworkInterfaceResourceController(GenericResourceController):
+    _resource_type = 'instance_network_interface'
+    Resource = ResourceSchema({
+        "name": six.text_type,
+        "instance_id": UUID,
+    })
+
+
 class InstanceResourceController(GenericResourceController):
     _resource_type = 'instance'
 
     Resource = ResourceSchema({
-        "flavor_id": int,
+        "flavor_id": six.text_type,
         "image_ref": six.text_type,
         "host": six.text_type,
         "display_name": six.text_type,
@@ -822,28 +935,28 @@ class GenericResourcesController(rest.RestController):
 
     @pecan.expose('json')
     def post(self):
-        body = deserialize(self.Resource)
+        body = deserialize_and_validate(self.Resource)
         target = {
             "resource_type": self._resource_type,
         }
         target.update(body)
         enforce("create resource", target)
         user, project = get_user_and_project()
-        body['metrics'] = convert_metric_list(
-            body.get('metrics', {}), user, project)
         rid = body['id']
         del body['id']
         try:
             resource = pecan.request.indexer.create_resource(
                 self._resource_type, rid, user, project,
                 **body)
-        except (ValueError, indexer.NoSuchMetric) as e:
+        except (ValueError,
+                indexer.NoSuchMetric,
+                indexer.NoSuchArchivePolicy) as e:
             abort(400, e)
         except indexer.ResourceAlreadyExists as e:
             abort(409, e)
         set_resp_location_hdr("/v1/resource/"
                               + self._resource_type + "/"
-                              + six.text_type(resource['id']))
+                              + six.text_type(resource.id))
         etag_set_headers(resource)
         pecan.response.status = 201
         return resource
@@ -852,6 +965,8 @@ class GenericResourcesController(rest.RestController):
     def get_all(self, **kwargs):
         details = get_details(kwargs)
         history = get_history(kwargs)
+        pagination_opts = get_pagination_options(
+            kwargs, RESOURCE_DEFAULT_PAGINATION)
 
         try:
             enforce("list all resource", {
@@ -868,11 +983,15 @@ class GenericResourcesController(rest.RestController):
             attr_filter = None
 
         try:
+            # FIXME(sileht): next API version should returns
+            # {'resources': [...], 'links': [ ... pagination rel ...]}
             return pecan.request.indexer.list_resources(
                 self._resource_type,
                 attribute_filter=attr_filter,
                 details=details,
-                history=history)
+                history=history,
+                **pagination_opts
+            )
         except indexer.IndexerException as e:
             abort(400, e)
 
@@ -882,6 +1001,20 @@ class SwiftAccountsResourcesController(GenericResourcesController):
     _resource_rest_class = SwiftAccountResourceController
 
 
+class InstanceDisksResourcesController(GenericResourcesController):
+    _resource_type = 'instance_disk'
+    _resource_rest_class = InstanceDiskResourceController
+
+    Resource = InstanceDiskResourceController.Resource
+
+
+class InstanceNetworkInterfacesResourcesController(GenericResourcesController):
+    _resource_type = 'instance_network_interface'
+    _resource_rest_class = InstanceNetworkInterfaceResourceController
+
+    Resource = InstanceNetworkInterfaceResourceController.Resource
+
+
 class InstancesResourcesController(GenericResourcesController):
     _resource_type = 'instance'
     _resource_rest_class = InstanceResourceController
@@ -929,8 +1062,8 @@ class ImageResourcesController(GenericResourcesController):
 
 
 class ResourcesController(rest.RestController):
-    resources_class_by_type = dict(
-        (ext.name, ext.plugin)
+    resources_ctrl_by_type = dict(
+        (ext.name, ext.plugin())
         for ext in extension.ExtensionManager(
             'gnocchi.controller.resources').extensions)
 
@@ -939,12 +1072,15 @@ class ResourcesController(rest.RestController):
         return dict(
             (type_name,
              pecan.request.application_url + '/v1/resource/' + type_name)
-            for type_name in self.resources_class_by_type.keys())
+            for type_name in self.resources_ctrl_by_type.keys())
 
-
-for resource_type, resource_class in (
-        ResourcesController.resources_class_by_type.items()):
-    setattr(ResourcesController, resource_type, resource_class())
+    @pecan.expose()
+    def _lookup(self, resource_type, *remainder):
+        ctrl = self.resources_ctrl_by_type.get(resource_type)
+        if ctrl:
+            return ctrl, remainder
+        else:
+            abort(404, indexer.UnknownResourceType(resource_type))
 
 
 def _ResourceSearchSchema(v):
@@ -982,12 +1118,14 @@ class SearchResourceTypeController(rest.RestController):
     @pecan.expose('json')
     def post(self, **kwargs):
         if pecan.request.body:
-            attr_filter = deserialize(self.ResourceSearchSchema)
+            attr_filter = deserialize_and_validate(self.ResourceSearchSchema)
         else:
             attr_filter = None
 
         details = get_details(kwargs)
         history = get_history(kwargs)
+        pagination_opts = get_pagination_options(
+            kwargs, RESOURCE_DEFAULT_PAGINATION)
 
         try:
             enforce("search all resource", {
@@ -1014,7 +1152,8 @@ class SearchResourceTypeController(rest.RestController):
                 self._resource_type,
                 attribute_filter=attr_filter,
                 details=details,
-                history=history)
+                history=history,
+                **pagination_opts)
         except indexer.IndexerException as e:
             abort(400, e)
 
@@ -1022,8 +1161,10 @@ class SearchResourceTypeController(rest.RestController):
 class SearchResourceController(rest.RestController):
     @pecan.expose()
     def _lookup(self, resource_type, *remainder):
-        # TODO(jd) Check that resource_type is valid
-        return SearchResourceTypeController(resource_type), remainder
+        if resource_type in ResourcesController.resources_ctrl_by_type:
+            return SearchResourceTypeController(resource_type), remainder
+        else:
+            abort(404, indexer.UnknownResourceType(resource_type))
 
 
 def _MetricSearchSchema(v):
@@ -1093,25 +1234,33 @@ class SearchMetricController(rest.RestController):
         if not pecan.request.body:
             abort(400, "No query specified in body")
 
-        query = deserialize(self.MetricSearchSchema)
+        query = deserialize_and_validate(self.MetricSearchSchema)
+
+        if start is not None:
+            try:
+                start = Timestamp(start)
+            except Exception:
+                abort(400, "Invalid value for start")
+
+        if stop is not None:
+            try:
+                stop = Timestamp(stop)
+            except Exception:
+                abort(400, "Invalid value for stop")
 
         try:
             return {
                 str(metric.id): values
                 for metric, values in six.iteritems(
                     pecan.request.storage.search_value(
-                        # NOTE(jd) Don't pass the archive policy as no
-                        # driver needs it for now
-                        [storage.Metric(str(metric['id']), None)
-                         for metric in metrics],
-                        query, start, stop, aggregation)
+                        metrics, query, start, stop, aggregation)
                 )
             }
         except storage.InvalidQuery as e:
             abort(400, e)
 
 
-class SearchController(rest.RestController):
+class SearchController(object):
     resource = SearchResourceController()
     metric = SearchMetricController()
 
@@ -1154,22 +1303,10 @@ class Aggregation(rest.RestController):
             arg_to_list(metric), start, stop, aggregation, needed_overlap)
 
 
-class V1Controller(rest.RestController):
-    search = SearchController()
-
-    archive_policy = ArchivePoliciesController()
-    archive_policy_rule = ArchivePolicyRulesController()
-    metric = MetricsController()
-    resource = ResourcesController()
-    aggregation = Aggregation()
-
-    _custom_actions = {
-        'capabilities': ['GET'],
-    }
-
+class CapabilityController(rest.RestController):
     @staticmethod
     @pecan.expose('json')
-    def get_capabilities():
+    def get():
         aggregation_methods = set(
             archive_policy.ArchivePolicy.VALID_AGGREGATION_METHODS)
         aggregation_methods.update(
@@ -1178,6 +1315,46 @@ class V1Controller(rest.RestController):
         return dict(aggregation_methods=aggregation_methods)
 
 
+class StatusController(rest.RestController):
+    @staticmethod
+    @pecan.expose('json')
+    def get():
+        enforce("get status", {})
+        report = pecan.request.storage.measures_report()
+        return {"storage": {"measures_to_process": report}}
+
+
+class V1Controller(object):
+
+    def __init__(self):
+        self.sub_controllers = {
+            "search": SearchController(),
+            "archive_policy": ArchivePoliciesController(),
+            "archive_policy_rule": ArchivePolicyRulesController(),
+            "metric": MetricsController(),
+            "resource": ResourcesController(),
+            "aggregation": Aggregation(),
+            "capabilities": CapabilityController(),
+            "status": StatusController(),
+        }
+        for name, ctrl in self.sub_controllers.items():
+            setattr(self, name, ctrl)
+
+    @pecan.expose('json')
+    def index(self):
+        return {
+            "version": "1.0",
+            "links": [
+                {"rel": "self",
+                 "href": pecan.request.application_url + "/v1"}
+            ] + [
+                {"rel": name,
+                 "href": pecan.request.application_url + "/v1/" + name}
+                for name in sorted(self.sub_controllers)
+            ]
+        }
+
+
 class RootController(object):
     v1 = V1Controller()
 
