@@ -200,5 +200,5 @@ class CephStorage(_carbonara.CarbonaraBasedStorage):
             if not data:
                 break
             content += data
-            offset += len(content)
+            offset += len(data)
         return content
